<?php

namespace Drupal\commerce_bpay_v2\PluginForm\OffsiteRedirect;

use Drupal;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\GeneratedUrl;
use Drupal\Core\Url;

/**
 * Предоставляет класс для оплаты за пределами сайта.
 *
 * Provide a buildConfigurationForm() method which calls buildRedirectForm()
 * with the right parameters.
 */
class BPayRedirectForm extends BasePaymentOffsiteForm
{

    const PRECISION = 2;

    const DEC_POINT = '.';

    const THOUSANDS_SEP = '';

    const ROUND_MODE = PHP_ROUND_HALF_UP;

    /**
     * Плагин шлюза.
     *
     * @var OffsitePaymentGatewayInterface
     */
    private $paymentGatewayPlugin;

    /**
     * {@inheritdoc}
     */
    public function buildConfigurationForm(array $form, FormStateInterface $form_state)
    {
        $form = parent::buildConfigurationForm($form, $form_state);

        /** @var PaymentInterface $payment */
        $payment = $this->entity;
        $this->paymentGatewayPlugin = $payment->getPaymentGateway()->getPlugin();

        $order = $payment->getOrder();
        $sandbox = $this->getConfiguration('mode');
        $redirect_url = $this->getDefaultSettings("{$sandbox}_uri");
        $json = [
            'merchantid' => $this->getConfiguration('merchant_id'),
            'amount' => $order->getTotalPrice()->getNumber(),
            'order_id' => $payment->getOrderId(),
            'description' => 'Оплата заказа №' . $order->id(),
            'lang' => 'ru',
            'method' => 'bpay',
            'fail_url' => $this->generateURL($order->id(), 'fail'),
            'success_url' => $this->generateURL($order->id(), 'success'),
            'callback_url' => $this->url('commerce_bpay_md.bpay_md_redirect_controller_notify'),
            'valute' => 498,
            'dtime' => date('Ymd H:i:s'),
            'getUrl' => '0',
        ];

        $json['params']['order_id'] = $order->id();

        $i = 0;
        foreach ($order->getItems() as $key => $order_item) {
            $product_variation = $order_item->getPurchasedEntity();
            $quantity = round($order_item->getQuantity(), 0);
            $json['params']['itemsPay'][$i]['title'] = $product_variation->getTitle();
            $json['params']['itemsPay'][$i]['price'] = $this->round($order_item->getUnitPrice()->getNumber());
            $json['params']['itemsPay'][$i]['quantity'] = $quantity;
            $i++;
        }
        $w = base64_encode(json_encode($json));

        $app = [
            'data' => $w,
            'key' => hash('sha256', $w . $this->getConfiguration('signature'))
        ];

        return $this->buildRedirectForm($form, $form_state, $redirect_url, $app, self::REDIRECT_POST);
    }

    /**
     * Получение конфигурации плагина.
     *
     * @param string $configuration
     *   Имя конфигурации.
     *
     * @return mixed
     *   Значение конфигурации.
     */
    private function getConfiguration($configuration)
    {
        $config = $this->paymentGatewayPlugin->getConfiguration()[$configuration];
        if (!$config) {
            return null;
        }
        return $this->paymentGatewayPlugin->getConfiguration()[$configuration];
    }

    /**
     * Получите настройки платежного шлюза по умолчанию.
     *
     * @param string|null $name
     *   Название настройки.
     *
     * @return array|ImmutableConfig|mixed|null
     *   Настройки по умолчанию.
     */
    private function getDefaultSettings( $name = NULL)
    {
        $config = Drupal::config('commerce_bpay_v2.settings');

        if ($name) {
            $cfg = $config->get("commerce_bpay_v2.{$name}");
            if (!$cfg) {
                return null;
            }
            return $cfg;
        }
        return null;
    }

    protected function generateURL( $order_id,  $ident)
    {
        $protocol = stripos($_SERVER["SERVER_PROTOCOL"], 'https') === 0 ? 'https' : 'http';
        return $protocol . '://' . $_SERVER['SERVER_NAME'] . '/bpay/' . $order_id . '/' . $ident;
    }

    /**
     * Создание URL.
     *
     * @param string $route
     *   Название маршрута.
     *
     * @return GeneratedUrl|string
     *   Url.
     */
    private function url( $route)
    {
        return Url::fromRoute($route, [], ['absolute' => TRUE])->toString();
    }

    /**
     * Round price.
     *
     * @param float $number
     *   Ценга.
     * @param int   $precision
     *   Точность.
     * @param int   $mode
     *   Режим округлени/.
     *
     * @return float
     *   Цена округленная.
     */
    private function round( $number, $precision = self::PRECISION, $mode = self::ROUND_MODE)
    {
        return (float)number_format(round($number, $precision, $mode), $precision, self::DEC_POINT, self::THOUSANDS_SEP);
    }
}
