<?php

namespace Drupal\commerce_bpay_v2\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides the Off-site Redirect payment Liqapy gateway.
 *
 * @CommercePaymentGateway(
 *   id = "bpay_v2_gateway",
 *   label = "BPay Payment (Off-site redirect)",
 *   display_label = "BPay Payment",
 *   forms = {
 *   "offsite-payment" =
 *   "Drupal\commerce_bpay_v2\PluginForm\OffsiteRedirect\BPayRedirectForm"
 *   },
 *   payment_method_types = {"credit_card"},
 *   modes = {
 *     "sandbox" = @Translation("Sandbox"),
 *     "live" = @Translation("Live")
 *   },
 * )
 */
class BPayPaymentGateway extends OffsitePaymentGatewayBase
{

    /**
     * {@inheritdoc}
     */
    public function defaultConfiguration()
    {
        $defaults = [
            'merchant_id' => '',
            'signature' => '',
        ];
        return $defaults + parent::defaultConfiguration();
    }

    /**
     * {@inheritdoc}
     */
    public function buildConfigurationForm($form, FormStateInterface $form_state)
    {
        $form = parent::buildConfigurationForm($form, $form_state);

        $form['merchant_id'] = [
            '#title' => $this->t('Merchant ID'),
            '#description' => $this->t('MerchantID интернет магазина, полученный при регистрации в качестве Merchant’a'),
            '#default_value' => $this->configuration['merchant_id'],
            '#type' => 'textfield',
            '#required' => TRUE,
        ];

        $form['signature'] = [
            '#title' => $this->t('Signature'),
            '#description' => $this->t('Signature интернет магазина, полученный при регистрации'),
            '#default_value' => $this->configuration['signature'],
            '#type' => 'textfield',
            '#required' => TRUE,
        ];
        return $form;
    }

    /**
     * {@inheritdoc}
     */
    public function validateConfigurationForm(&$form, FormStateInterface $form_state)
    {
        parent::validateConfigurationForm($form, $form_state);

        $values = $form_state->getValue($form['#parents']);
        $required_fields = [
            'merchant_id',
            'signature',
        ];

        foreach ($required_fields as $key) {
            if (empty($values[$key])) {
                $message = t('Service is not configured for use. Please contact an administrator to resolve this issue.');
                $this->messenger()->addError($message);
                return FALSE;
            }
        }
    }

    public function onReturn(OrderInterface $order, Request $request)
    {
        $data = $request->request->get('data');
        $jsondata = json_decode(base64_decode($data));


        $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
        $payment = $payment_storage->loadByRemoteId($jsondata->transid);
        if ($payment) {
            $transition = $payment->getState()->getWorkflow()->getTransition($this->mapRemoteStateToTransition('completed'));
            $payment->getState()->applyTransition($transition);
            $payment->setRemoteState('completed');
            $payment->save();
        } else {
            $payment = $payment_storage->create([
                'state' => 'completed',
                'amount' => $order->getTotalPrice(),
                'payment_gateway' => $this->entityId,
                'payment_gateway_mode' => 'live',
                'order_id' => $order->id(),
                'remote_id' => $jsondata->transid,
                'remote_state' => 'completed',
                'locked' => 0
            ]);
            $payment->save();
        }

        if ($payment->getState()->value === 'completed') {
            $order_s = $payment->getOrder();
            $transition = $order_s->getState()->getWorkflow()->getTransition('place');
            $order_s->getState()->applyTransition($transition);
            $order_s->unlock();
            $order_s->save();
        }

    }

    protected function mapRemoteStateToTransition($remote_state)
    {
        $states = [
            'completed' => 'capture',
            'cancelled' => 'void',
            'expired' => 'expire',
            'refunded' => 'refund',
            'partial_refunded' => 'partially_refund',
        ];

        return $states[$remote_state];
    }

    /**
     * {@inheritdoc}
     */
    public function submitConfigurationForm(&$form, FormStateInterface $form_state)
    {
        parent::submitConfigurationForm($form, $form_state);

        if (!$form_state->getErrors()) {
            $values = $form_state->getValue($form['#parents']);
            $this->configuration['merchant_id'] = $values['merchant_id'];
            $this->configuration['signature'] = $values['signature'];
        }
    }
}
